@@ -14,61 +14,38 @@
 
 import os
 import sys
-import time
-# BBB
-from os.path import realpath  # NOQA
 
 from Acquisition import aq_base
 from Acquisition import aq_parent
+from zope.deferredimport import deprecated
+
+
+deprecated(
+    'Please import from zope.datetime.'
+    ' This backwards compatibility import will go away in Zope 6.',
+    weekday_abbr='zope.datetime:weekday_abbr',
+    weekday_full='zope.datetime:weekday_full',
+    monthname='zope.datetime:monthname',
+    iso8601_date='zope.datetime:iso8601_date',
+    rfc850_date='zope.datetime:rfc850_date',
+    rfc1123_date='zope.datetime:rfc1123_date',
+)
+
+deprecated(
+    'Please import from os.path.'
+    ' This backwards compatibility import will go away in Zope 6.',
+    realpath='os.path:realpath',
+)
+
+deprecated(
+    'Please import time directly.'
+    ' This backwards compatibility import will go away in Zope 6.',
+    time='time',
+)
 
 
 attrget = getattr
 
-# These are needed because the various date formats below must
-# be in english per the RFCs. That means we can't use strftime,
-# which is affected by different locale settings.
-weekday_abbr = ['Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat', 'Sun']
-weekday_full = ['Monday', 'Tuesday', 'Wednesday', 'Thursday',
-                'Friday', 'Saturday', 'Sunday']
-monthname = [None, 'Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun',
-             'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec']
-
-
-def iso8601_date(ts=None):
-    # Return an ISO 8601 formatted date string, required
-    # for certain DAV properties.
-    # '2000-11-10T16:21:09-08:00
-    if ts is None:
-        ts = time.time()
-    return time.strftime('%Y-%m-%dT%H:%M:%SZ', time.gmtime(ts))
-
-
-def rfc850_date(ts=None):
-    # Return an HTTP-date formatted date string.
-    # 'Friday, 10-Nov-00 16:21:09 GMT'
-    if ts is None:
-        ts = time.time()
-    year, month, day, hh, mm, ss, wd, y, z = time.gmtime(ts)
-    return "%s, %02d-%3s-%2s %02d:%02d:%02d GMT" % (
-        weekday_full[wd],
-        day, monthname[month],
-        str(year)[2:],
-        hh, mm, ss)
-
-
-def rfc1123_date(ts=None):
-    # Return an RFC 1123 format date string, required for
-    # use in HTTP Date headers per the HTTP 1.1 spec.
-    # 'Fri, 10 Nov 2000 16:21:09 GMT'
-    if ts is None:
-        ts = time.time()
-    year, month, day, hh, mm, ss, wd, y, z = time.gmtime(ts)
-    return "%s, %02d %3s %4d %02d:%02d:%02d GMT" % (
-        weekday_abbr[wd],
-        day, monthname[month],
-        year,
-        hh, mm, ss)
-
 
 def absattr(attr, callable=callable):
     # Return the absolute value of an attribute,
@@ -85,6 +62,11 @@ def is_acquired(ob, hasattr=hasattr, aq_base=aq_base, absattr=absattr):
     # Note that this method is misnamed since parents can (and do)
     # spoof it. It is not a true measure of whether something is
     # acquired, it relies on the parent's parent-ness exclusively
+    import warnings
+    warnings.warn(
+        "The function `is_acquired` is deprecated "
+        "and will be removed in Zope 6.",
+        DeprecationWarning)
     if not hasattr(ob, '__parent__'):
         # We can't be acquired if we don't have an __parent__
         return 0
