@@ -22,16 +22,18 @@ import transaction
 from AccessControl import ClassSecurityInfo
 from AccessControl.class_init import InitializeClass
 from AccessControl.Permission import ApplicationDefaultPermissions
+from AccessControl.Permissions import view_management_screens
 from Acquisition import aq_base
 from App import FactoryDispatcher
 from App.ApplicationManager import ApplicationManager
 from App.ProductContext import ProductContext
+from App.version_txt import getZopeVersion
 from DateTime import DateTime
-from OFS import bbb
 from OFS.FindSupport import FindSupport
 from OFS.metaconfigure import get_packages_to_initialize
 from OFS.metaconfigure import package_initialized
 from OFS.userfolder import UserFolder
+from webdav.NullResource import NullResource
 from zExceptions import Forbidden
 from zExceptions import Redirect as RedirectException
 from zope.interface import implementer
@@ -42,11 +44,6 @@ from .interfaces import IApplication
 from .misc_ import Misc_
 
 
-if bbb.HAS_ZSERVER:
-    from webdav.NullResource import NullResource
-else:
-    NullResource = bbb.NullResource
-
 LOG = getLogger('Application')
 
 APP_MANAGER = None
@@ -130,6 +127,9 @@ class Application(ApplicationDefaultPermissions, Folder.Folder, FindSupport):
         if NullResource is not None and method not in ('GET', 'POST'):
             return NullResource(self, name, REQUEST).__of__(self)
 
+        if method not in ('GET', 'POST'):
+            return NullResource(self, name, REQUEST).__of__(self)
+
         # Waaa. unrestrictedTraverse calls us with a fake REQUEST.
         # There is probably a better fix for this.
         try:
@@ -141,16 +141,33 @@ class Application(ApplicationDefaultPermissions, Folder.Folder, FindSupport):
         """Utility function to return current date/time"""
         return DateTime(*args)
 
-    if bbb.HAS_ZSERVER:
-        def DELETE(self, REQUEST, RESPONSE):
-            """Delete a resource object."""
-            self.dav__init(REQUEST, RESPONSE)
-            raise Forbidden('This resource cannot be deleted.')
+    @security.protected(view_management_screens)
+    def ZopeVersion(self, major=False):
+        """Utility method to return the Zope version
 
-        def MOVE(self, REQUEST, RESPONSE):
-            """Move a resource to a new location."""
-            self.dav__init(REQUEST, RESPONSE)
-            raise Forbidden('This resource cannot be moved.')
+        Restricted to ZMI to prevent information disclosure
+        """
+        zversion = getZopeVersion()
+        if major:
+            return zversion.major
+        else:
+            version = '%s.%s.%s' % (zversion.major,
+                                    zversion.minor,
+                                    zversion.micro)
+            if zversion.status:
+                version += '.%s%s' % (zversion.status, zversion.release)
+
+            return version
+
+    def DELETE(self, REQUEST, RESPONSE):
+        """Delete a resource object."""
+        self.dav__init(REQUEST, RESPONSE)
+        raise Forbidden('This resource cannot be deleted.')
+
+    def MOVE(self, REQUEST, RESPONSE):
+        """Move a resource to a new location."""
+        self.dav__init(REQUEST, RESPONSE)
+        raise Forbidden('This resource cannot be moved.')
 
     def absolute_url(self, relative=0):
         """The absolute URL of the root object is BASE1 or "/".
@@ -200,7 +217,7 @@ def initialize(app):
 
 
 class AppInitializer(object):
-    """ Initialze an Application object (called at startup) """
+    """ Initialize an Application object (called at startup) """
 
     def __init__(self, app):
         self.app = (app,)
