@@ -14,21 +14,17 @@
 """
 
 import sys
+
 import ExtensionClass
 import zope.pagetemplate.pagetemplate
-from zope.pagetemplate.pagetemplate import PTRuntimeError
-from zope.pagetemplate.pagetemplate import PageTemplateTracebackSupplement
-from zope.tales.expressions import SimpleModuleImporter
+from Acquisition import aq_base
+from Acquisition import aq_inner
+from Acquisition import aq_parent
 from Products.PageTemplates.Expressions import getEngine
+from zope.pagetemplate.pagetemplate import PageTemplateTracebackSupplement
+from zope.pagetemplate.pagetemplate import PTRuntimeError
+from zope.tales.expressions import SimpleModuleImporter
 
-# Newer versions of zope.pagetemplate have this symbol as an instance method
-try:
-    from zope.pagetemplate.pagetemplate import _error_start
-except ImportError:
-    _error_start = getattr(
-        zope.pagetemplate.pagetemplate.PageTemplate, '_error_start', 
-        '<!-- Page Template Diagnostics')
-    
 
 class PageTemplate(ExtensionClass.Base,
                    zope.pagetemplate.pagetemplate.PageTemplate):
@@ -43,14 +39,14 @@ class PageTemplate(ExtensionClass.Base,
              'request': None,
              'modules': SimpleModuleImporter(),
              }
-        parent = getattr(self, 'aq_parent', None)
+        parent = getattr(self, '__parent__', None)
         if parent is not None:
             c['here'] = parent
             c['context'] = parent
-            c['container'] = self.aq_inner.aq_parent
+            c['container'] = aq_parent(aq_inner(self))
             while parent is not None:
                 self = parent
-                parent = getattr(self, 'aq_parent', None)
+                parent = getattr(self, '__parent__', None)
             c['root'] = self
         return c
 
@@ -62,10 +58,11 @@ class PageTemplate(ExtensionClass.Base,
     def pt_macros(self):
         self._cook_check()
         if self._v_errors:
-            __traceback_supplement__ = (PageTemplateTracebackSupplement, self, {})
-            raise PTRuntimeError, (
+            __traceback_supplement__ = (
+                PageTemplateTracebackSupplement, self, {})
+            raise PTRuntimeError(
                 'Page Template %s has errors: %s' % (
-                self.id, self._v_errors
+                    self.id, self._v_errors
                 ))
         return self._v_macros
 
@@ -83,9 +80,9 @@ class PageTemplate(ExtensionClass.Base,
             showtal = sourceAnnotations = False
         if source:
             showtal = True
-        return super(PageTemplate, self).pt_render(c, source=source, sourceAnnotations=sourceAnnotations,
-                   showtal=showtal)
-
+        return super(PageTemplate, self).pt_render(
+            c, source=source, sourceAnnotations=sourceAnnotations,
+            showtal=showtal)
 
     def pt_errors(self, namespace={}, check_macro_expansion=None):
         # The check_macro_expansion argument is added for
@@ -97,11 +94,11 @@ class PageTemplate(ExtensionClass.Base,
             return err
         try:
             self.pt_render(source=True, extra_context=namespace)
-        except:
+        except Exception:
             return ('Macro expansion failed', '%s: %s' % sys.exc_info()[:2])
 
     def __call__(self, *args, **kwargs):
-        if not kwargs.has_key('args'):
+        if 'args' not in kwargs:
             kwargs['args'] = args
         return self.pt_render(extra_context={'options': kwargs})
 
@@ -112,20 +109,20 @@ class PageTemplate(ExtensionClass.Base,
                 return self._text
             try:
                 return self.pt_render(source=True)
-            except:
+            except Exception:
                 return ('%s\n Macro expansion failed\n %s\n-->\n%s' %
-                        (_error_start, "%s: %s" % sys.exc_info()[:2],
-                         self._text) )
+                        (self._error_start, "%s: %s" % sys.exc_info()[:2],
+                         self._text))
 
-        return ('%s\n %s\n-->\n%s' % (_error_start,
+        return ('%s\n %s\n-->\n%s' % (self._error_start,
                                       '\n '.join(self._v_errors),
                                       self._text))
 
     # convenience method for the ZMI which allows to explicitly
-    # specify the HTMLness of a template.  The old Zope 2
+    # specify the HTMLness of a template. The old Zope 2
     # implementation had this as well, but arguably on the wrong class
     # (this should be a ZopePageTemplate thing if at all)
     def html(self):
-        if not hasattr(getattr(self, 'aq_base', self), 'is_html'):
+        if not hasattr(aq_base(self), 'is_html'):
             return self.content_type == 'text/html'
         return self.is_html
