@@ -3,7 +3,7 @@ import sys
 
 import Zope2
 from AccessControl.SecurityManagement import newSecurityManager
-from AccessControl.SpecialUsers import system as user
+from AccessControl.users import system as user
 from Testing.makerequest import makerequest
 from Zope2.Startup.run import make_wsgi_app
 from zope.globalrequest import setRequest
